#include "config.hpp"

#include <iostream>
#include <toml++/toml.h>

static sk::config read(const toml::table &config) {
    return sk::config{
        sk::queue_config{
            config["queue"]["pull"].value_or("tcp://localhost:5557"),
            config["queue"]["push"].value_or("tcp://localhost:5558"),
            config["queue"]["threads"].value_or(1u),
        },
        sk::runner_config{
            config["runner"]["timeout"].value_or(2u),
        },
    };
}

sk::config sk::config::read_or_default(const std::filesystem::path &path, bool expect_present) {
    std::ifstream t(path);
    if (!t) {
        if (errno == ENOENT && !expect_present) {
            std::cout << "Using default config\n";
        } else {
            std::cerr << "Failed to open config file " << path << ": " << strerror(errno) << "\n";
        }
        return read(toml::table{});
    }
    std::stringstream buffer;
    buffer << t.rdbuf();
    auto config = toml::parse(buffer.str(), path);
    return read(config);
}
