#pragma once

#include "config.hpp"
#include "program.hpp"
#include <mutex>
#include <string>
#include <vector>

namespace sk {
struct [[nodiscard]] run_result {
    std::string out;
    std::string err;
    int exit_code;
};

struct [[nodiscard]] active_job {
    std::string job_id;
    pid_t pid;
};

enum class runner_backend { BubbleWrap, Docker };

class runner {
    runner_backend backend;
    int timeout;
    std::vector<active_job> active_jobs;
    std::mutex active_jobs_mutex;

  public:
    runner(runner_backend backend, const runner_config &config);
    run_result run_blocking(const program &program);
    bool kill_active(const std::string &jobId);
    void exit_active_jobs();

  private:
    void exit(const active_job &job);
};
}
